package net.javainthebox.heapindicator;

import java.awt.BorderLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.lang.management.MemoryPoolMXBean;
import java.lang.management.MemoryUsage;
import java.lang.reflect.UndeclaredThrowableException;
import java.util.Collection;
import javax.swing.JPanel;
import javax.swing.Timer;

import com.sun.tools.visualvm.application.Application;
import com.sun.tools.visualvm.charts.ChartFactory;
import com.sun.tools.visualvm.charts.SimpleXYChartDescriptor;
import com.sun.tools.visualvm.charts.SimpleXYChartSupport;
import com.sun.tools.visualvm.core.datasource.DataSource;
import com.sun.tools.visualvm.core.datasupport.DataRemovedListener;
import com.sun.tools.visualvm.core.options.GlobalPreferences;
import com.sun.tools.visualvm.core.ui.DataSourceViewPlugin;
import com.sun.tools.visualvm.core.ui.components.DataViewComponent;
import com.sun.tools.visualvm.tools.jmx.JmxModel;
import com.sun.tools.visualvm.tools.jmx.JmxModelFactory;
import com.sun.tools.visualvm.tools.jmx.JvmMXBeans;
import com.sun.tools.visualvm.tools.jmx.JvmMXBeansFactory;

public class HeapView extends DataSourceViewPlugin {

    // JMX をカプセル化する JmxMobel オブジェクト
    private JmxModel jmx;
    
    // メモリ領域の名前
    private String poolName;

    // メモリ領域に関する MXBean
    private MemoryPoolMXBean mxbean;

    // チャート
    private SimpleXYChartSupport chartSupport;

    // データ更新用配列
    private long[] values = new long[2];

    // 値更新用のタイマ
    private Timer timer;
    private int pollingTime;

    // タブの順番
    private int index;

    public HeapView(Application application, String poolName, int index) {
        super(application);

        this.poolName = poolName;
        this.index = index;
        
        // JMX を用いてアプリケーションとの接続を行なう
        jmx = JmxModelFactory.getJmxModelFor(application);
        
        // JMX の接続が切れたら、タイマーを停止させる
        jmx.addPropertyChangeListener(new PropertyChangeListener() {
            @Override
            public void propertyChange(PropertyChangeEvent event) {
                if ("connectionState".equals(event.getPropertyName())
                        && event.getNewValue() == JmxModel.ConnectionState.DISCONNECTED) {
                    timer.stop();
                }
             }
        });

        // プラットフォームMBeanServerに相当するJvmMXBeansオブジェクトを取得
        JvmMXBeans mxbeans = JvmMXBeansFactory.getJvmMXBeans(jmx);
        Collection<MemoryPoolMXBean> poolMXBeans = mxbeans.getMemoryPoolMXBeans();
        for (MemoryPoolMXBean poolMXBean: poolMXBeans) {
            String name = poolMXBean.getName();
            if (name.contains(poolName)) {
                mxbean = poolMXBean;
            }
        }
    }
    
    @Override
    public DataViewComponent.DetailsView createView(int location) {
        if (location == DataViewComponent.TOP_RIGHT) {

            // メモリ領域のチャート作成
            chartSupport = createChartSupport(mxbean);

            // タイマの初期化
            initTimer();
                
            // チャートをパネルに貼る
            JPanel chartPanel = new JPanel();
            chartPanel.setLayout(new BorderLayout());
            chartPanel.add(chartSupport.getChart(), BorderLayout.CENTER);

            // サブタブを生成
            return new DataViewComponent.DetailsView(poolName,
                                                     null,
                                                     index,
                                                     chartPanel,
                                                     null);
        } else {
            return null;
        }
    }

    private void updateValues() {
        // Eden使用量の更新
        if (mxbean != null) {
            updateValue(mxbean, chartSupport);
        }
    }

    public void initTimer() {
        // 周期的に値を更新する
        ActionListener task = new ActionListener () {
            @Override
            public void actionPerformed(ActionEvent event) {
                if (jmx.getConnectionState() == JmxModel.ConnectionState.CONNECTED) {
                    // JMX との通信が確立している場合
                    try {
                        updateValues();
                    } catch (UndeclaredThrowableException ex) {
                        // 情報取得中に通信が切れた場合
                    }
                }
            }
        };
        timer = new Timer(pollingTime * 1000, task);
        timer.setDelay(pollingTime * 1000);
        timer.start();

        // 終了時にタイマを停止させる
        DataSource dataSource = getDataSource();
        dataSource.notifyWhenRemoved(new DataRemovedListener() {
            @Override
            public void dataRemoved(Object arg0) {
                timer.stop();
            }
        });
    }
    
    private void updateValue(MemoryPoolMXBean mxBean, SimpleXYChartSupport chartSupport) {
        MemoryUsage usage = mxBean.getUsage();

        // チャートに値を追加
        values[0] = usage.getCommitted();
        values[1] = usage.getUsed();
        chartSupport.addValues(System.currentTimeMillis(), values);
    }

    private SimpleXYChartSupport createChartSupport(MemoryPoolMXBean mxBean) {
 
        // メニューバーの[ツール]-[オプション]で設定した値を
        // 取得するためのオブジェクト
        GlobalPreferences preferences = GlobalPreferences.sharedInstance();

        // 値を更新する周期の設定
        pollingTime = preferences.getMonitoredDataPoll();
        // 値を保持しておく時間 (分)
        int cacheMin = preferences.getMonitoredDataCache();
        // 値を保持しておくサイズ
        int chartCache = cacheMin * 60 / pollingTime;

        // 初期状態でのy軸の高さ
        long yMargin = mxBean.getUsage().getCommitted();

        // y軸の単位がByteなので、
        // byteメソッドでSimpleXYChartDescriptorオブジェクトを作成
        SimpleXYChartDescriptor chartDescriptor =
                    SimpleXYChartDescriptor.bytes(yMargin, false, chartCache);

        // チャートに描画する項目
        chartDescriptor.addLineFillItems("Committed", "Used");

        // SimpleXYChartSupportオブジェクトの生成
        return ChartFactory.createSimpleXYChart(chartDescriptor);
    }
}
